@@ -29,8 +29,9 @@ use futures::{
 
 use leaky_bucket::RateLimiter;
 use librqbit_core::{
+    crate_version,
     hash_id::Id20,
-    peer_id::generate_peer_id,
+    peer_id::generate_azereus_style,
     spawn_utils::{spawn, spawn_with_cancel},
 };
 use parking_lot::RwLock;
@@ -265,7 +266,7 @@ impl RecursiveRequest<RecursiveRequestCallbacksFindNodes> {
         let request_one = |id, addr, depth| {
             req.request_one(id, addr, depth)
                 .map_err(|e| {
-                    debug!("error: {e:?}");
+                    debug!("error: {e:#}");
                     e
                 })
                 .instrument(error_span!(
@@ -341,7 +342,7 @@ impl RecursiveRequest<RecursiveRequestCallbacksGetPeers> {
                                 Ok(n) if n < 8 => REQUERY_INTERVAL / 8 * (n as u32),
                                 Ok(_) => REQUERY_INTERVAL,
                                 Err(e) => {
-                                    error!("error in get_peers_root(): {e:?}");
+                                    error!("error in get_peers_root(): {e:#}");
                                     return Err::<(), anyhow::Error>(e);
                                 }
                             };
@@ -359,7 +360,7 @@ impl RecursiveRequest<RecursiveRequestCallbacksGetPeers> {
                             let (id, addr, depth) = addr.unwrap();
                             futs.push(
                                 this.request_one(id, addr, depth)
-                                    .map_err(|e| debug!("error: {e:?}"))
+                                    .map_err(|e| debug!("error: {e:#}"))
                                     .instrument(error_span!("addr", addr=addr.to_string()))
                             );
                         }
@@ -402,10 +403,13 @@ impl<C: RecursiveRequestCallbacks> RecursiveRequest<C> {
             self.callbacks.on_request_start(self, id, addr);
         }
 
-        let response = self.dht.request(self.request.clone(), addr).await.map(|r| {
-            self.mark_node_responded(addr, &r);
-            r
-        });
+        let response = self
+            .dht
+            .request(self.request.clone(), addr)
+            .await
+            .inspect(|r| {
+                self.mark_node_responded(addr, r);
+            });
         if let Some(id) = id {
             self.callbacks.on_request_end(self, id, addr, &response);
         }
@@ -967,9 +971,10 @@ impl DhtWorker {
             loop {
                 interval.tick().await;
                 let mut found = 0;
+                let now = Instant::now();
                 for node in self.dht.routing_table.read().iter() {
                     if matches!(
-                        node.status(),
+                        node.status(now),
                         NodeStatus::Questionable | NodeStatus::Unknown
                     ) {
                         found += 1;
@@ -996,7 +1001,7 @@ impl DhtWorker {
                             },
                             Err(e) => {
                                 self.dht.routing_table.write().mark_error(&id);
-                                debug!("error: {e:?}");
+                                debug!("error: {e:#}");
                             }
                         }
                     }.instrument(error_span!("ping", addr=addr.to_string())))
@@ -1033,7 +1038,7 @@ impl DhtWorker {
                 )
                 .unwrap();
                 if let Err(e) = socket.send_to(&buf, addr).await {
-                    debug!("error sending to {addr}: {e:?}");
+                    debug!("error sending to {addr}: {e:#}");
                     if let Some(tid) = our_tid {
                         self.on_send_error(tid, addr, e.into());
                     }
@@ -1164,7 +1169,9 @@ impl DhtState {
                 .context("cannot determine UDP listen addr")?;
             info!("DHT listening on {:?}", listen_addr);
 
-            let peer_id = config.peer_id.unwrap_or_else(generate_peer_id);
+            let peer_id = config
+                .peer_id
+                .unwrap_or_else(|| (generate_azereus_style(*b"rQ", crate_version!())));
             info!("starting up DHT with peer id {:?}", peer_id);
             let bootstrap_addrs = config
                 .bootstrap_addrs
@@ -1194,17 +1201,12 @@ impl DhtState {
         .boxed()
     }
 
-    #[inline(never)]
     pub fn get_peers(
         self: &Arc<Self>,
         info_hash: Id20,
         announce_port: Option<u16>,
-    ) -> anyhow::Result<RequestPeersStream> {
-        Ok(RequestPeersStream::new(
-            self.clone(),
-            info_hash,
-            announce_port,
-        ))
+    ) -> RequestPeersStream {
+        RequestPeersStream::new(self.clone(), info_hash, announce_port)
     }
 
     pub fn listen_addr(&self) -> SocketAddr {
