@@ -5,7 +5,11 @@ use librqbit_core::lengths::{ChunkInfo, Lengths, ValidPieceIndex};
 use peer_binary_protocol::Piece;
 use tracing::{debug, trace};
 
-use crate::type_aliases::{FilePriorities, OpenedFiles, BF};
+use crate::{
+    bitv::{BitV, BoxBitV},
+    file_info::FileInfo,
+    type_aliases::{FileInfos, FilePriorities, BF, BS},
+};
 
 pub struct ChunkTracker {
     // This forms the basis of a "queue" to pull from.
@@ -23,12 +27,15 @@ pub struct ChunkTracker {
     chunk_status: BF,
 
     // These are the pieces that we actually have, fully checked and downloaded.
-    have: BF,
+    have: BoxBitV,
 
     // The pieces that the user selected. This doesn't change unless update_only_files
     // was called.
     selected: BF,
 
+    // How many bytes do we have per each file.
+    per_file_bytes: Vec<u64>,
+
     lengths: Lengths,
 
     // Quick to retrieve stats, that MUST be in sync with the BFs
@@ -64,7 +71,7 @@ impl HaveNeededSelected {
 // Comput the have-status of chunks.
 //
 // Save as "have_pieces", but there's one bit per chunk (not per piece).
-fn compute_chunk_have_status(lengths: &Lengths, have_pieces: &BF) -> anyhow::Result<BF> {
+fn compute_chunk_have_status(lengths: &Lengths, have_pieces: &BS) -> anyhow::Result<BF> {
     if have_pieces.len() < lengths.total_pieces() as usize {
         anyhow::bail!(
             "bug: have_pieces.len() < lengths.total_pieces(); {} < {}",
@@ -72,6 +79,7 @@ fn compute_chunk_have_status(lengths: &Lengths, have_pieces: &BF) -> anyhow::Res
             lengths.total_pieces()
         );
     }
+
     let required_size = lengths.chunk_bitfield_bytes();
     let vec = vec![0u8; required_size];
     let mut chunk_bf = BF::from_boxed_slice(vec.into_boxed_slice());
@@ -92,15 +100,19 @@ fn compute_chunk_have_status(lengths: &Lengths, have_pieces: &BF) -> anyhow::Res
     Ok(chunk_bf)
 }
 
-fn compute_queued_pieces_unchecked(have_pieces: &BF, selected_pieces: &BF) -> BF {
+fn compute_queued_pieces_unchecked(have_pieces: &BS, selected_pieces: &BS) -> BF {
     // it's needed ONLY if it's selected and we don't have it.
     use core::ops::BitAnd;
     use core::ops::Not;
 
-    have_pieces.clone().not().bitand(selected_pieces)
+    have_pieces
+        .to_bitvec()
+        .not()
+        .bitand(selected_pieces)
+        .into_boxed_bitslice()
 }
 
-fn compute_queued_pieces(have_pieces: &BF, selected_pieces: &BF) -> anyhow::Result<BF> {
+fn compute_queued_pieces(have_pieces: &BS, selected_pieces: &BS) -> anyhow::Result<BF> {
     if have_pieces.len() != selected_pieces.len() {
         anyhow::bail!(
             "have_pieces.len() != selected_pieces.len(), {} != {}",
@@ -115,6 +127,7 @@ fn compute_queued_pieces(have_pieces: &BF, selected_pieces: &BF) -> anyhow::Resu
     ))
 }
 
+#[derive(Debug)]
 pub enum ChunkMarkingResult {
     PreviouslyCompleted,
     NotCompleted,
@@ -124,41 +137,59 @@ pub enum ChunkMarkingResult {
 impl ChunkTracker {
     pub fn new(
         // Have pieces are the ones we have already downloaded and verified.
-        have_pieces: BF,
+        have_pieces: BoxBitV,
         // Selected pieces are the ones the user has selected
         selected_pieces: BF,
         lengths: Lengths,
+        file_infos: &FileInfos,
     ) -> anyhow::Result<Self> {
-        let needed_pieces = compute_queued_pieces(&have_pieces, &selected_pieces)
+        let needed_pieces = compute_queued_pieces(have_pieces.as_slice(), &selected_pieces)
             .context("error computing needed pieces")?;
 
         // TODO: ideally this needs to be a list based on needed files, e.g.
         // last needed piece for each file. But let's keep simple for now.
 
         let mut ct = Self {
-            chunk_status: compute_chunk_have_status(&lengths, &have_pieces)
+            chunk_status: compute_chunk_have_status(&lengths, have_pieces.as_slice())
                 .context("error computing chunk status")?,
             queue_pieces: needed_pieces,
             selected: selected_pieces,
             lengths,
             have: have_pieces,
             hns: HaveNeededSelected::default(),
+            per_file_bytes: vec![0; file_infos.len()],
         };
+        ct.recalculate_per_file_bytes(file_infos);
         ct.hns = ct.calc_hns();
         Ok(ct)
     }
 
+    fn recalculate_per_file_bytes(&mut self, file_infos: &FileInfos) {
+        for (slot, fi) in self.per_file_bytes.iter_mut().zip(file_infos.iter()) {
+            *slot = fi
+                .piece_range
+                .clone()
+                .filter(|p| self.have.as_slice()[*p as usize])
+                .map(|id| {
+                    self.lengths
+                        .size_of_piece_in_file(id, fi.offset_in_torrent, fi.len)
+                })
+                .sum();
+        }
+    }
+
     pub fn get_lengths(&self) -> &Lengths {
         &self.lengths
     }
 
-    pub fn get_have_pieces(&self) -> &BF {
-        &self.have
+    pub fn get_have_pieces(&self) -> &dyn BitV {
+        &*self.have
     }
 
-    pub fn get_selected_pieces(&self) -> &BF {
-        &self.selected
+    pub fn get_have_pieces_mut(&mut self) -> &mut dyn BitV {
+        &mut *self.have
     }
+
     pub fn reserve_needed_piece(&mut self, index: ValidPieceIndex) {
         self.queue_pieces.set(index.get() as usize, false)
     }
@@ -172,7 +203,7 @@ impl ChunkTracker {
         for piece in self.lengths.iter_piece_infos() {
             let id = piece.piece_index.get() as usize;
             let len = piece.len as u64;
-            let is_have = self.have[id];
+            let is_have = self.have.as_slice()[id];
             let is_selected = self.selected[id];
             let is_needed = is_selected && !is_have;
             hns.have_bytes += len * (is_have as u64);
@@ -185,47 +216,33 @@ impl ChunkTracker {
     pub(crate) fn iter_queued_pieces<'a>(
         &'a self,
         file_priorities: &'a FilePriorities,
-        opened_files: &'a OpenedFiles,
+        file_infos: &'a FileInfos,
     ) -> impl Iterator<Item = ValidPieceIndex> + 'a {
         file_priorities
             .iter()
-            .filter_map(|p| opened_files.get(*p))
-            .filter(|f| !f.approx_is_finished())
-            .flat_map(|f| f.iter_piece_priorities())
+            .filter_map(|p| Some((*p, file_infos.get(*p)?)))
+            .filter(|(id, f)| self.per_file_bytes[*id] != f.len)
+            .flat_map(|(_id, f)| f.iter_piece_priorities())
             .filter(|id| self.queue_pieces[*id])
             .filter_map(|id| id.try_into().ok())
             .filter_map(|id| self.lengths.validate_piece_index(id))
     }
 
-    // None if wrong chunk
-    // true if did something
-    // false if didn't do anything
-    pub fn mark_chunk_request_cancelled(
-        &mut self,
-        index: ValidPieceIndex,
-        _chunk: u32,
-    ) -> Option<bool> {
-        if *self.have.get(index.get() as usize)? {
-            return Some(false);
-        }
-        // This will trigger the requesters to re-check each chunk in this piece.
-        let chunk_range = self.lengths.chunk_range(index);
-        if !self.chunk_status.get(chunk_range)?.all() {
-            self.queue_pieces.set(index.get() as usize, true);
-        }
-        Some(true)
+    pub(crate) fn is_piece_have(&self, id: ValidPieceIndex) -> bool {
+        self.have.as_slice()[id.get() as usize]
     }
 
     pub fn mark_piece_broken_if_not_have(&mut self, index: ValidPieceIndex) {
         if self
             .have
+            .as_slice()
             .get(index.get() as usize)
             .map(|r| *r)
             .unwrap_or_default()
         {
             return;
         }
-        debug!("remarking piece={} as broken", index);
+        debug!("marking piece={} as broken", index);
         self.queue_pieces.set(index.get() as usize, true);
         if let Some(s) = self.chunk_status.get_mut(self.lengths.chunk_range(index)) {
             s.fill(false);
@@ -234,8 +251,8 @@ impl ChunkTracker {
 
     pub fn mark_piece_downloaded(&mut self, idx: ValidPieceIndex) {
         let id = idx.get() as usize;
-        if !self.have[id] {
-            self.have.set(id, true);
+        if !self.have.as_slice()[id] {
+            self.have.as_slice_mut().set(id, true);
             let len = self.lengths.piece_length(idx) as u64;
             self.hns.have_bytes += len;
             if self.selected[id] {
@@ -246,6 +263,7 @@ impl ChunkTracker {
 
     pub fn is_chunk_ready_to_upload(&self, chunk: &ChunkInfo) -> bool {
         self.have
+            .as_slice()
             .get(chunk.piece_index.get() as usize)
             .map(|b| *b)
             .unwrap_or(false)
@@ -321,7 +339,8 @@ impl ChunkTracker {
                 current_piece_remaining -= TryInto::<u32>::try_into(shift)?;
 
                 if current_piece_remaining == 0 {
-                    let current_piece_have = self.have[current_piece.piece_index.get() as usize];
+                    let current_piece_have =
+                        self.have.as_slice()[current_piece.piece_index.get() as usize];
                     if current_piece_have {
                         have_bytes += current_piece.len as u64;
                     }
@@ -367,15 +386,50 @@ impl ChunkTracker {
         self.hns = res;
         Ok(res)
     }
+
+    pub(crate) fn get_selected_pieces(&self) -> &BF {
+        &self.selected
+    }
+
+    pub fn is_file_finished(&self, file_info: &FileInfo) -> bool {
+        self.have
+            .as_slice()
+            .get(file_info.piece_range_usize())
+            .map(|r| r.all())
+            .unwrap_or(true)
+    }
+
+    pub(crate) fn is_finished(&self) -> bool {
+        self.get_hns().finished()
+    }
+
+    pub fn per_file_have_bytes(&self) -> &[u64] {
+        &self.per_file_bytes
+    }
+
+    // Returns remaining bytes
+    pub fn update_file_have_on_piece_completed(
+        &mut self,
+        piece_id: ValidPieceIndex,
+        file_id: usize,
+        file_info: &FileInfo,
+    ) -> u64 {
+        let diff_have = self.lengths.size_of_piece_in_file(
+            piece_id.get(),
+            file_info.offset_in_torrent,
+            file_info.len,
+        );
+        self.per_file_bytes[file_id] += diff_have;
+        file_info.len.saturating_sub(self.per_file_bytes[file_id])
+    }
 }
 
 #[cfg(test)]
 mod tests {
+    use librqbit_core::{constants::CHUNK_SIZE, lengths::Lengths};
     use std::collections::HashSet;
 
-    use librqbit_core::{constants::CHUNK_SIZE, lengths::Lengths};
-
-    use crate::{chunk_tracker::HaveNeededSelected, type_aliases::BF};
+    use crate::{bitv::BitV, chunk_tracker::HaveNeededSelected, type_aliases::BF};
 
     use super::{compute_chunk_have_status, ChunkTracker};
 
@@ -497,7 +551,13 @@ mod tests {
         let initial_selected = BF::from_boxed_slice(vec![u8::MAX; bf_len].into_boxed_slice());
 
         // Initially, we need all files and all pieces.
-        let mut ct = ChunkTracker::new(initial_have.clone(), initial_selected.clone(), l).unwrap();
+        let mut ct = ChunkTracker::new(
+            initial_have.clone().into_dyn(),
+            initial_selected.clone(),
+            l,
+            &Default::default(),
+        )
+        .unwrap();
 
         // Select all file, no changes.
         assert_eq!(
@@ -509,7 +569,7 @@ mod tests {
                 needed_bytes: total_len,
             }
         );
-        assert_eq!(ct.have, initial_have);
+        assert_eq!(ct.have.as_slice(), initial_have.as_bitslice());
         assert_eq!(ct.queue_pieces, initial_selected);
 
         // Select only the first file.
