@@ -2,21 +2,23 @@ use std::{collections::HashSet, sync::Arc};
 
 use crate::{
     chunk_tracker::{ChunkTracker, HaveNeededSelected},
-    type_aliases::OpenedFiles,
+    type_aliases::FileStorage,
 };
 
-use super::ManagedTorrentInfo;
+use super::{streaming::TorrentStreams, ManagedTorrentShared, TorrentMetadata};
 
 pub struct TorrentStatePaused {
-    pub(crate) info: Arc<ManagedTorrentInfo>,
-    pub(crate) files: OpenedFiles,
+    pub(crate) shared: Arc<ManagedTorrentShared>,
+    pub(crate) metadata: Arc<TorrentMetadata>,
+    pub(crate) files: FileStorage,
     pub(crate) chunk_tracker: ChunkTracker,
+    pub(crate) streams: Arc<TorrentStreams>,
 }
 
 impl TorrentStatePaused {
     pub(crate) fn update_only_files(&mut self, only_files: &HashSet<usize>) -> anyhow::Result<()> {
         self.chunk_tracker
-            .update_only_files(self.info.info.iter_file_lengths()?, only_files)?;
+            .update_only_files(self.metadata.info.iter_file_lengths()?, only_files)?;
         Ok(())
     }
 
