@@ -1,6 +1,6 @@
 use anyhow::Context;
 
-use crate::{constants::CHUNK_SIZE, torrent_metainfo::TorrentMetaV1Info};
+use crate::constants::CHUNK_SIZE;
 
 pub fn last_element_size<T>(total_length: T, piece_length: T) -> T
 where
@@ -67,11 +67,15 @@ impl ValidPieceIndex {
     pub const fn get(&self) -> u32 {
         self.0
     }
+    pub const fn get_usize(&self) -> usize {
+        self.0 as usize
+    }
 }
 
 impl Lengths {
+    #[cfg(any(feature = "sha1-crypto-hash", feature = "sha1-ring"))]
     pub fn from_torrent<ByteBuf: AsRef<[u8]>>(
-        torrent: &TorrentMetaV1Info<ByteBuf>,
+        torrent: &crate::torrent_metainfo::TorrentMetaV1Info<ByteBuf>,
     ) -> anyhow::Result<Lengths> {
         let total_length = torrent.iter_file_lengths()?.sum();
         Lengths::new(total_length, torrent.piece_length)
@@ -154,11 +158,7 @@ impl Lengths {
     }
 
     // A helper to iterate over pieces in a file.
-    pub(crate) fn iter_pieces_within_offset(
-        &self,
-        offset_bytes: u64,
-        len: u64,
-    ) -> std::ops::Range<u32> {
+    pub fn iter_pieces_within_offset(&self, offset_bytes: u64, len: u64) -> std::ops::Range<u32> {
         // Validation and correction
         let offset_bytes = offset_bytes.min(self.total_length);
         let end_bytes = (offset_bytes + len).min(self.total_length);
@@ -262,6 +262,35 @@ impl Lengths {
 
         end.saturating_sub(offset)
     }
+
+    pub fn compute_current_piece(
+        self: &Lengths,
+        file_pos: u64,
+        file_torrent_abs_offset: u64,
+    ) -> Option<CurrentPiece> {
+        let dpl = self.default_piece_length();
+
+        let abs_pos = file_torrent_abs_offset + file_pos;
+        let piece_id = abs_pos / dpl as u64;
+        let piece_id: u32 = piece_id.try_into().ok()?;
+
+        let piece_id = self.validate_piece_index(piece_id)?;
+        let piece_len = self.piece_length(piece_id);
+        let piece_offset = (abs_pos / dpl as u64).try_into().ok()?;
+        Some(CurrentPiece {
+            id: piece_id,
+            piece_offset,
+            piece_remaining: (piece_len as u64 - (abs_pos % dpl as u64))
+                .try_into()
+                .ok()?,
+        })
+    }
+}
+
+pub struct CurrentPiece {
+    pub id: ValidPieceIndex,
+    pub piece_remaining: u32,
+    pub piece_offset: u32,
 }
 
 #[cfg(test)]
