@@ -5,9 +5,11 @@
 pub mod extended;
 
 use bincode::Options;
-use buffers::{ByteBuf, ByteBufOwned};
+use buffers::{ByteBuf, ByteBufOwned, ByteBufT};
 use byteorder::{ByteOrder, BE};
+use bytes::Bytes;
 use clone_to_owned::CloneToOwned;
+use extended::PeerExtendedMessageIds;
 use librqbit_core::{constants::CHUNK_SIZE, hash_id::Id20, lengths::ChunkInfo};
 use serde::{Deserialize, Serialize};
 
@@ -43,8 +45,12 @@ const MSGID_PIECE: u8 = 7;
 const MSGID_CANCEL: u8 = 8;
 const MSGID_EXTENDED: u8 = 20;
 
+pub const EXTENDED_UT_METADATA_KEY: &[u8] = b"ut_metadata";
 pub const MY_EXTENDED_UT_METADATA: u8 = 3;
 
+pub const EXTENDED_UT_PEX_KEY: &[u8] = b"ut_pex";
+pub const MY_EXTENDED_UT_PEX: u8 = 1;
+
 #[derive(Debug)]
 pub enum MessageDeserializeError {
     NotEnoughData(usize, &'static str),
@@ -75,24 +81,36 @@ pub fn serialize_piece_preamble(chunk: &ChunkInfo, mut buf: &mut [u8]) -> usize
 }
 
 #[derive(Debug)]
-pub struct Piece<ByteBuf> {
+pub struct Piece<B> {
     pub index: u32,
     pub begin: u32,
-    pub block: ByteBuf,
+    pub block: B,
 }
 
-impl<ByteBuf> Piece<ByteBuf>
+impl<B: CloneToOwned> CloneToOwned for Piece<B> {
+    type Target = Piece<B::Target>;
+
+    fn clone_to_owned(&self, within_buffer: Option<&Bytes>) -> Self::Target {
+        Piece {
+            index: self.index,
+            begin: self.begin,
+            block: self.block.clone_to_owned(within_buffer),
+        }
+    }
+}
+
+impl<B> Piece<B>
 where
-    ByteBuf: AsRef<[u8]>,
+    B: AsRef<[u8]>,
 {
-    pub fn from_data<T>(index: u32, begin: u32, block: T) -> Piece<ByteBuf>
+    pub fn from_data<T>(index: u32, begin: u32, block: T) -> Piece<B>
     where
-        ByteBuf: From<T>,
+        B: From<T>,
     {
         Piece {
             index,
             begin,
-            block: ByteBuf::from(block),
+            block: B::from(block),
         }
     }
 
@@ -103,13 +121,13 @@ where
         buf.copy_from_slice(self.block.as_ref());
         self.block.as_ref().len() + 8
     }
-    pub fn deserialize<'a>(buf: &'a [u8]) -> Piece<ByteBuf>
+    pub fn deserialize<'a>(buf: &'a [u8]) -> Piece<B>
     where
-        ByteBuf: From<&'a [u8]> + 'a,
+        B: From<&'a [u8]> + 'a,
     {
         let index = byteorder::BigEndian::read_u32(&buf[0..4]);
         let begin = byteorder::BigEndian::read_u32(&buf[4..8]);
-        let block = ByteBuf::from(&buf[8..]);
+        let block = B::from(&buf[8..]);
         Piece {
             index,
             begin,
@@ -145,7 +163,7 @@ impl std::fmt::Display for MessageDeserializeError {
                 len_prefix,
             } => write!(
                 f,
-                "error deserializing {name} (msg_id={msg_id}, len_prefix={len_prefix}): {error:?}"
+                "error deserializing {name} (msg_id={msg_id}, len_prefix={len_prefix}): {error:#}"
             ),
             MessageDeserializeError::Other(e) => write!(f, "{e}"),
         }
@@ -168,7 +186,7 @@ impl From<anyhow::Error> for MessageDeserializeError {
 }
 
 #[derive(Debug)]
-pub enum Message<ByteBuf: std::hash::Hash + Eq> {
+pub enum Message<ByteBuf: ByteBufT> {
     Request(Request),
     Cancel(Request),
     Bitfield(ByteBuf),
@@ -185,8 +203,8 @@ pub enum Message<ByteBuf: std::hash::Hash + Eq> {
 pub type MessageBorrowed<'a> = Message<ByteBuf<'a>>;
 pub type MessageOwned = Message<ByteBufOwned>;
 
-pub type BitfieldBorrowed<'a> = &'a bitvec::slice::BitSlice<u8, bitvec::order::Lsb0>;
-pub type BitfieldOwned = bitvec::vec::BitVec<u8, bitvec::order::Lsb0>;
+pub type BitfieldBorrowed<'a> = &'a bitvec::slice::BitSlice<u8, bitvec::order::Msb0>;
+pub type BitfieldOwned = bitvec::vec::BitVec<u8, bitvec::order::Msb0>;
 
 pub struct Bitfield<'a> {
     pub data: BitfieldBorrowed<'a>,
@@ -194,28 +212,28 @@ pub struct Bitfield<'a> {
 
 impl<ByteBuf> CloneToOwned for Message<ByteBuf>
 where
-    ByteBuf: CloneToOwned + std::hash::Hash + Eq,
-    <ByteBuf as CloneToOwned>::Target: std::hash::Hash + Eq,
+    ByteBuf: ByteBufT,
+    <ByteBuf as CloneToOwned>::Target: ByteBufT,
 {
     type Target = Message<<ByteBuf as CloneToOwned>::Target>;
 
-    fn clone_to_owned(&self) -> Self::Target {
+    fn clone_to_owned(&self, within_buffer: Option<&Bytes>) -> Self::Target {
         match self {
             Message::Request(req) => Message::Request(*req),
             Message::Cancel(req) => Message::Cancel(*req),
-            Message::Bitfield(b) => Message::Bitfield(b.clone_to_owned()),
+            Message::Bitfield(b) => Message::Bitfield(b.clone_to_owned(within_buffer)),
             Message::Choke => Message::Choke,
             Message::Unchoke => Message::Unchoke,
             Message::Interested => Message::Interested,
             Message::Piece(piece) => Message::Piece(Piece {
                 index: piece.index,
                 begin: piece.begin,
-                block: piece.block.clone_to_owned(),
+                block: piece.block.clone_to_owned(within_buffer),
             }),
             Message::KeepAlive => Message::KeepAlive,
             Message::Have(v) => Message::Have(*v),
             Message::NotInterested => Message::NotInterested,
-            Message::Extended(e) => Message::Extended(e.clone_to_owned()),
+            Message::Extended(e) => Message::Extended(e.clone_to_owned(within_buffer)),
         }
     }
 }
@@ -228,7 +246,7 @@ impl<'a> Bitfield<'a> {
     }
 }
 
-impl<'a> std::fmt::Debug for Bitfield<'a> {
+impl std::fmt::Debug for Bitfield<'_> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("Bitfield")
             .field("_ones", &self.data.count_ones())
@@ -239,7 +257,7 @@ impl<'a> std::fmt::Debug for Bitfield<'a> {
 
 impl<ByteBuf> Message<ByteBuf>
 where
-    ByteBuf: AsRef<[u8]> + std::hash::Hash + Eq + Serialize,
+    ByteBuf: ByteBufT,
 {
     pub fn len_prefix_and_msg_id(&self) -> (u32, u8) {
         match self {
@@ -262,7 +280,7 @@ where
     pub fn serialize(
         &self,
         out: &mut Vec<u8>,
-        extended_handshake_ut_metadata: &dyn Fn() -> Option<u8>,
+        peer_extended_messages: &dyn Fn() -> PeerExtendedMessageIds,
     ) -> anyhow::Result<usize> {
         let (lp, msg_id) = self.len_prefix_and_msg_id();
 
@@ -312,7 +330,7 @@ where
                 Ok(msg_len)
             }
             Message::Extended(e) => {
-                e.serialize(out, extended_handshake_ut_metadata)?;
+                e.serialize(out, peer_extended_messages)?;
                 let msg_size = out.len();
                 // no fucking idea why +1, but I tweaked that for it all to match up
                 // with real messages.
@@ -573,9 +591,9 @@ where
 {
     type Target = Handshake<<B as CloneToOwned>::Target>;
 
-    fn clone_to_owned(&self) -> Self::Target {
+    fn clone_to_owned(&self, within_buffer: Option<&Bytes>) -> Self::Target {
         Handshake {
-            pstr: self.pstr.clone_to_owned(),
+            pstr: self.pstr.clone_to_owned(within_buffer),
             reserved: self.reserved,
             info_hash: self.info_hash,
             peer_id: self.peer_id,
@@ -622,7 +640,7 @@ mod tests {
     fn test_extended_serialize() {
         let msg = Message::Extended(ExtendedMessage::Handshake(ExtendedHandshake::new()));
         let mut out = Vec::new();
-        msg.serialize(&mut out, &|| None).unwrap();
+        msg.serialize(&mut out, &Default::default).unwrap();
         dbg!(out);
     }
 
@@ -638,7 +656,7 @@ mod tests {
         let (msg, size) = MessageBorrowed::deserialize(&buf).unwrap();
         assert_eq!(size, buf.len());
         let mut write_buf = Vec::new();
-        msg.serialize(&mut write_buf, &|| None).unwrap();
+        msg.serialize(&mut write_buf, &Default::default).unwrap();
         if buf != write_buf {
             {
                 use std::io::Write;
